#include "generator.h"

/******************************************************************************/
ExamGenerator::ExamGenerator():
_Max(0),
_Index(0),
_DigitLimit(0),
_HighValue(0),
_Seed(0),
_Random(NULL),
_QuestionType(0),
_GivenQuestionType(0),
_QuestionStr(NULL),
_FinalResult(0),
_AnswerStr(NULL),
_Dividend(0),
_Divisor(0),
_Quotient(0),
_Remainder(0),
_RemainderPercent(10),
_Addend(0),
_Adder(0),
_Sum(0),
_Subtrahend(0),
_Subtractor(0),
_Difference(0),
_Multiplicand(0),
_Multiplier(0),
_Product(0),
_AddendSet(false),
_SubtrahendSet(false),
_NoRemSet(false),
_AppendSet(false),
_Multipart(false),
_SplitOn(0)
{
  _Random = new Randomizer<long double>;
  _QuestionStr = new char[256];
  _AnswerStr = new char[256];
}

/******************************************************************************/
ExamGenerator::ExamGenerator(int Max_):
_Max(Max_),
_Index(0),
_DigitLimit(0),
_HighValue(0),
_Seed(0),
_Random(NULL),
_QuestionType(0),
_GivenQuestionType(0),
_QuestionStr(NULL),
_FinalResult(0),
_AnswerStr(NULL),
_Dividend(0),
_Divisor(0),
_Quotient(0),
_Remainder(0),
_RemainderPercent(10),
_Addend(0),
_Adder(0),
_Sum(0),
_Subtrahend(0),
_Subtractor(0),
_Difference(0),
_Multiplicand(0),
_Multiplier(0),
_Product(0),
_AddendSet(false),
_SubtrahendSet(false),
_NoRemSet(false),
_AppendSet(false),
_Multipart(false),
_SplitOn(0)
{
  _Random = new Randomizer<long double>;
  _QuestionStr = new char[256];
  _AnswerStr = new char[256];
}

/******************************************************************************/
ExamGenerator::~ExamGenerator()
{
  Clear();
}

/******************************************************************************/
void ExamGenerator::Clear()
{
  delete[] _AnswerStr;
  delete[] _QuestionStr;
  delete _Random;

  _AnswerStr = NULL;
  _QuestionStr = NULL;
}

/******************************************************************************/
bool ExamGenerator::IsPrime(int Num_, int& Div_) const
{
  int Limit_ = int(::floor(::sqrt(Num_)));
  int x, div_, mod_;

  for (x = 2; x <= Limit_ && x < Num_; x++)
  {
    div_ = Num_ / x;
    mod_ = Num_ % x;

    if (mod_ == 0 && (1 < div_ && div_ < Num_))
    {
      Div_ = x;
      return false;
    }
  }

  return true;
}

/******************************************************************************/
void ExamGenerator::Reduce()
{
  int Limit_ = int(::floor(::sqrt(_ResDenom)));
  bool done = false;
  int x;

  while (!done)
  {
    done = true;
    for (x = 2; x <= Limit_ && x <= _ResDenom && x <= _ResNumer; x++)
      if (_ResDenom % x == 0 && _ResNumer % x == 0)
      {
        _ResDenom /= x;
        _ResNumer /= x;
        done = false;
      }
  }
}

/******************************************************************************/
void ExamGenerator::FillQuestion(bool Append_)
{
  char OpChar_ = ' ';
  char SplitOpChar_ = ' ';
  int Operand2_;
  int Operand1_;
  int Operand2a_;
  int Operand2b_;
  int Temp_ = 0;
  int x, z, z1, z2;
  int y = Append_ ? ::strlen(_QuestionStr):0;
  z = 0;

  if (_QuestionType == ADDITION)
  {
    Operand1_ = _Addend;
    Operand2_ = _Adder;
    OpChar_ = '+';

    if (_SplitOn == 2)
    {
      Operand2a_ = (_SplitOp == ADDITION) ? (_Adder - _SplitVal):_SplitVal;
      Operand2b_ = (_SplitOp == ADDITION) ? _SplitVal:(_SplitVal - _Adder);
      SplitOpChar_ = (_SplitOp == ADDITION) ? '+':'-';
    }
  }
  else if (_QuestionType == SUBTRACTION)
  {
    Operand1_ = _Subtrahend;
    Operand2_ = _Subtractor;
    OpChar_ = '-';

    if (_SplitOn == 2)
    {
      Operand2a_ = (_SplitOp == ADDITION) ? (_Subtractor - _SplitVal):_SplitVal;
      Operand2b_ = (_SplitOp == ADDITION) ? _SplitVal:(_SplitVal - _Subtractor);
      SplitOpChar_ = (_SplitOp == ADDITION) ? '+':'-';
    }
  }
  else if (_QuestionType == MULTIPLICATION)
  {
    Operand1_ = _Multiplicand;
    Operand2_ = _Multiplier;
    OpChar_ = 'x';

    if (_SplitOn == 1)
    {
      Operand2a_ = (_SplitOp == ADDITION) ? (_Multiplier - _SplitVal):_SplitVal;
      Operand2b_ = (_SplitOp == ADDITION) ? _SplitVal:(_SplitVal - _Multiplier);
      SplitOpChar_ = (_SplitOp == ADDITION) ? '+':'-';
    }
  }
  else if (_QuestionType == DIVISION)
  {
    Operand1_ = _Dividend;
    Operand2_ = _Divisor;
    OpChar_ = '/';

    if (_SplitOn == 1)
    {
      Operand2a_ = (_SplitOp == ADDITION) ? (_Divisor - _SplitVal):_SplitVal;
      Operand2b_ = (_SplitOp == ADDITION) ? _SplitVal:(_SplitVal - _Divisor);
      SplitOpChar_ = (_SplitOp == ADDITION) ? '+':'-';
    }
  }

  if (!Append_ && Operand1_ > 9 && _HighValue >= 10)
  {
    for (x = _HighValue; x > 1; x /= 10)
      if (Operand1_ / x)
      {
        _QuestionStr[y++] = '0' + Operand1_ / x;
        Temp_ = (Operand1_ / x) * x;
        Operand1_ -= Temp_;
      }
      else if (y)
        _QuestionStr[y++] = '0';

    _QuestionStr[y++] = '0' + Operand1_ % 10;
  }
  else if (!Append_)
    _QuestionStr[y++] = '0' + Operand1_;

  _QuestionStr[y++] = ' ';
  _QuestionStr[y++] = OpChar_;
  _QuestionStr[y++] = ' ';

  Temp_ = 0;
  z = 0;
  z1 = z2 = 0;

  if (_SplitOn == 0)
  {
    if (Operand2_ > 9 && _HighValue >= 10)
    {
      for (x = _HighValue; x > 1; x /= 10)
        if (Operand2_ / x)
        {
          ++z;
          _QuestionStr[y++] = '0' + Operand2_ / x;
          Temp_ = (Operand2_ / x) * x;
          Operand2_ -= Temp_;
        }
        else if (z)
          _QuestionStr[y++] = '0';

      _QuestionStr[y++] = '0' + Operand2_ % 10;
    }
    else
      _QuestionStr[y++] = '0' + Operand2_;
  }
  else if (_SplitOn == 1 || _SplitOn == 2)
  {
    _QuestionStr[y++] = '(';
    if (Operand2a_ > 9 && _HighValue >= 10)
    {
      for (x = _HighValue; x > 1; x /= 10)
        if (Operand2a_ / x)
        {
          ++z1;
          _QuestionStr[y++] = '0' + Operand2a_ / x;
          Temp_ = (Operand2a_ / x) * x;
          Operand2a_ -= Temp_;
        }
        else if (z1)
          _QuestionStr[y++] = '0';

      _QuestionStr[y++] = '0' + Operand2a_ % 10;
    }
    else
      _QuestionStr[y++] = '0' + Operand2a_;

    _QuestionStr[y++] = ' ';
    _QuestionStr[y++] = SplitOpChar_;
    _QuestionStr[y++] = ' ';

    if (Operand2b_ > 9 && _HighValue >= 10)
    {
      for (x = _HighValue; x > 1; x /= 10)
        if (Operand2b_ / x)
        {
          ++z2;
          _QuestionStr[y++] = '0' + Operand2b_ / x;
          Temp_ = (Operand2b_ / x) * x;
          Operand2b_ -= Temp_;
        }
        else if (z2)
          _QuestionStr[y++] = '0';

      _QuestionStr[y++] = '0' + Operand2b_ % 10;
    }
    else
      _QuestionStr[y++] = '0' + Operand2b_;

    _QuestionStr[y++] = ')';
  }

  _QuestionStr[y] = 0;
}

/******************************************************************************/
void ExamGenerator::FillFracQuestion(bool Append_)
{
  char OpChar_ = ' ';
  int Denom1_ = _Denom1;
  int Numer1_ = _Numer1;
  int Denom2_ = _Denom2;
  int Numer2_ = _Numer2;
  int Temp_ = 0;
  int x, z1, z2, z3;
  int y = Append_ ? ::strlen(_QuestionStr):0;
  z1 = 0;

  if (_QuestionType == FRACTION_ADD)
    OpChar_ = '+';
  else if (_QuestionType == FRACTION_SUB)
    OpChar_ = '-';
  else if (_QuestionType == FRACTION_MULT)
    OpChar_ = 'x';
  else if (_QuestionType == FRACTION_DIV)
    OpChar_ = '/';

  if (Numer1_ > 9 && _HighValue >= 10)
  {
    for (x = _HighValue; x > 1; x /= 10)
      if (Numer1_ / x)
      {
        _QuestionStr[y++] = '0' + Numer1_ / x;
        Temp_ = (Numer1_ / x) * x;
        Numer1_ -= Temp_;
      }
      else if (y)
        _QuestionStr[y++] = '0';

    _QuestionStr[y++] = '0' + Numer1_ % 10;
  }
  else
    _QuestionStr[y++] = '0' + Numer1_;

  _QuestionStr[y++] = '/';
  Temp_ = 0;
  z1 = 0;

  if (Denom1_ > 9 && _HighValue >= 10)
  {
    for (x = _HighValue; x > 1; x /= 10)
      if (Denom1_ / x)
      {
        ++z1;
        _QuestionStr[y++] = '0' + Denom1_ / x;
        Temp_ = (Denom1_ / x) * x;
        Denom1_ -= Temp_;
      }
      else if (z1)
        _QuestionStr[y++] = '0';

    _QuestionStr[y++] = '0' + Denom1_ % 10;
  }
  else
    _QuestionStr[y++] = '0' + Denom1_;

  _QuestionStr[y++] = ' ';
  _QuestionStr[y++] = OpChar_;
  _QuestionStr[y++] = ' ';
  z2 = 0;

  if (Numer2_ > 9 && _HighValue >= 10)
  {
    for (x = _HighValue; x > 1; x /= 10)
      if (Numer2_ / x)
      {
        ++z2;
        _QuestionStr[y++] = '0' + Numer2_ / x;
        Temp_ = (Numer2_ / x) * x;
        Numer2_ -= Temp_;
      }
      else if (z2)
        _QuestionStr[y++] = '0';

    _QuestionStr[y++] = '0' + Numer2_ % 10;
  }
  else
    _QuestionStr[y++] = '0' + Numer2_;

  _QuestionStr[y++] = '/';
  Temp_ = 0;
  z3 = 0;

  if (Denom2_ > 9 && _HighValue >= 10)
  {
    for (x = _HighValue; x > 1; x /= 10)
      if (Denom2_ / x)
      {
        ++z3;
        _QuestionStr[y++] = '0' + Denom2_ / x;
        Temp_ = (Denom2_ / x) * x;
        Denom2_ -= Temp_;
      }
      else if (z3)
        _QuestionStr[y++] = '0';

    _QuestionStr[y++] = '0' + Denom2_ % 10;
  }
  else
    _QuestionStr[y++] = '0' + Denom2_;

  _QuestionStr[y] = 0;
}

/******************************************************************************/
void ExamGenerator::FillAnswer()
{
  int FinalResult_ = _FinalResult;
  int FinalResult2_ = _FinalResult2;
  int Temp_ = 0;
  int x, y, z;
  y = 0;

  if (FinalResult_ > 9 && _HighValue >= 10)
  {
    for (x = _HighValue; x > 1; x /= 10)
      if (FinalResult_ / x)
      {
        _AnswerStr[y++] = '0' + FinalResult_ / x;
        Temp_ = (FinalResult_ / x) * x;
        FinalResult_ -= Temp_;
      }
      else if (y)
        _AnswerStr[y++] = '0';

    _AnswerStr[y++] = '0' + FinalResult_ % 10;
  }
  else
    _AnswerStr[y++] = '0' + FinalResult_;

  if (FinalResult2_)
  {
    _AnswerStr[y++] = ',';
    _AnswerStr[y] = 0;

    strcat(_AnswerStr, "R=");
    y = ::strlen(_AnswerStr);
    z = 0;

    if (FinalResult2_ > 9 && _HighValue >= 10)
    {
      for (x = _HighValue; x > 1; x /= 10)
        if (FinalResult2_ / x)
        {
          ++z;
          _AnswerStr[y++] = '0' + FinalResult2_ / x;
          Temp_ = (FinalResult2_ / x) * x;
          FinalResult2_ -= Temp_;
        }
        else if (z)
          _AnswerStr[y++] = '0';

      _AnswerStr[y++] = '0' + FinalResult2_ % 10;
    }
    else
      _AnswerStr[y++] = '0' + FinalResult2_;
  }

  _AnswerStr[y] = 0;
}

/******************************************************************************/
void ExamGenerator::FillFracAnswer()
{
  int ResDenom_ = _ResDenom;
  int ResNumer_ = _ResNumer;
  int Temp_ = 0;
  int x, y, z;
  y = 0;
  z = 0;

  if (ResNumer_ > 9 && _HighValue >= 10)
  {
    for (x = _HighValue; x > 1; x /= 10)
      if (ResNumer_ / x)
      {
        _AnswerStr[y++] = '0' + ResNumer_ / x;
        Temp_ = (ResNumer_ / x) * x;
        ResNumer_ -= Temp_;
      }
      else if (y)
        _AnswerStr[y++] = '0';

    _AnswerStr[y++] = '0' + ResNumer_ % 10;
  }
  else
    _AnswerStr[y++] = '0' + ResNumer_;

  _AnswerStr[y++] = '/';
  Temp_ = 0;
  z = 0;

  if (ResDenom_ > 9 && _HighValue >= 10)
  {
    for (x = _HighValue; x > 1; x /= 10)
      if (ResDenom_ / x)
      {
        ++z;
        _AnswerStr[y++] = '0' + ResDenom_ / x;
        Temp_ = (ResDenom_ / x) * x;
        ResDenom_ -= Temp_;
      }
      else if (z)
        _AnswerStr[y++] = '0';

    _AnswerStr[y++] = '0' + ResDenom_ % 10;
  }
  else
    _AnswerStr[y++] = '0' + ResDenom_;

  _AnswerStr[y] = 0;
}

/******************************************************************************/
void ExamGenerator::SetAddend(int Add_)
{
  _Addend = Add_;
  _AddendSet = true;
}

/******************************************************************************/
void ExamGenerator::SetSubtrahend(int Sub_)
{
  _Subtrahend = Sub_;
  _SubtrahendSet = true;
}

/******************************************************************************/
void ExamGenerator::GenerateProblems(int Max_, int Type_, int DigitLimit_)
{
  _Index = 0;
  _Max = Max_;
  _QuestionType = Type_;

  _DigitLimit = DigitLimit_;
  _HighValue = int(::ceil(::pow(10L, _DigitLimit)));

  _Seed = time(NULL);
  srand(_Seed);
  _Random->SetSeed(_Seed);
}

/******************************************************************************/
bool ExamGenerator::RemainderAllowed(int Val_) const
{
  return (Val_ <= _RemainderPercent);
}

/******************************************************************************/
void ExamGenerator::SetDivRemainderPercentage(int Val_)
{
  _RemainderPercent = Val_ ? Val_:10;
  --_RemainderPercent;
}

/******************************************************************************/
const char* ExamGenerator::GiveQuestion()
{
  if (_Index >= _Max)
    return NULL;

  _FinalResult = 0;
  _FinalResult2 = 0;
  int IsMixed_ = 0;
  int SavedSplit_ = 0;

  if (_QuestionType == MIXED)
  {
    _GivenQuestionType = _QuestionType = _Random->GiveRandom(MIXED);
    IsMixed_ = MIXED;
  }

  if (_QuestionType == ADDITION)
  {
    bool RestorePart_ = true;
    if (_Multipart)
      RestorePart_ = false;

    if (!_AddendSet)
    {
      do _Sum = _Random->GiveRandom(_HighValue);
      while (_Sum == 0);

      _Addend = _Random->GiveRandom(_Sum);
    }
    else
    {
      if (_Addend < _HighValue - 1)
      {
        do _Sum = _Random->GiveRandom(_HighValue - _Addend);
        while (_Sum == 0);

        _Sum += _Addend;
      }
      else
        _Sum = _HighValue - 1;
    }

    _Adder = _Sum - _Addend;
    if (_SplitOn == 2)
    {
      if (_SplitOp == ADDITION)
        _SplitVal = _Random->GiveRandom(_Adder);
      else if (_SplitOp == SUBTRACTION)
      {
        _SplitVal = _Random->GiveRandom(_HighValue - _Adder);
        _SplitVal += _Adder;
      }
    }
    else if (_SplitOn == 1)
    {
      _SplitOn = 0;
      SavedSplit_ = 1;
    }

    _FinalResult = _Sum;
    _FinalResult2 = 0;
    FillQuestion(_AppendSet);

    if (SavedSplit_)
      _SplitOn = SavedSplit_;

    if (RestorePart_)
      _Multipart = false;
  }
  else if (_QuestionType == SUBTRACTION)
  {
    bool RestorePart_ = true;
    if (_Multipart)
      RestorePart_ = false;

    if (!_SubtrahendSet)
    {
      do _Subtrahend = _Random->GiveRandom(_HighValue);
      while (_Subtrahend == 0);
    }

    _Subtractor = _Random->GiveRandom(_Subtrahend);
    _Difference = _Subtrahend - _Subtractor;
    if (_SplitOn == 2)
    {
      if (_SplitOp == ADDITION)
        _SplitVal = _Random->GiveRandom(_Subtractor);
      else if (_SplitOp == SUBTRACTION)
      {
        _SplitVal = _Random->GiveRandom(_HighValue - _Subtractor);
        _SplitVal += _Subtractor;
      }
    }
    else if (_SplitOn == 1)
    {
      _SplitOn = 0;
      SavedSplit_ = 1;
    }

    _FinalResult = _Difference;
    _FinalResult2 = 0;
    FillQuestion(_AppendSet);

    if (SavedSplit_)
      _SplitOn = SavedSplit_;

    if (RestorePart_)
      _Multipart = false;
  }
  else if (_QuestionType == MULTIPLICATION)
  {
    bool RestorePart_ = true;
    if (_Multipart)
      RestorePart_ = false;

    do
    {
      _Product = _Random->GiveRandom(_HighValue);
      if (_Product > 1)
      {
        do _Multiplicand = _Random->GiveRandom(_Product);
        while (_Multiplicand == 0);
      }
      else
        _Multiplicand = 1;

      _Remainder = _Product % _Multiplicand;
    }
    while (_Product == 0 || _Remainder > 0);

    _Multiplier = _Product / _Multiplicand;
    if (_SplitOn == 1)
    {
      if (_SplitOp == ADDITION)
        _SplitVal = _Random->GiveRandom(_Multiplier);
      else if (_SplitOp == SUBTRACTION)
      {
        _SplitVal = _Random->GiveRandom(_HighValue - _Multiplier);
        _SplitVal += _Multiplier;
      }
    }
    else if (_SplitOn == 2)
    {
      _SplitOn = 0;
      SavedSplit_ = 2;
    }

    _FinalResult = _Product;
    _FinalResult2 = 0;
    FillQuestion(_AppendSet);

    if (SavedSplit_)
      _SplitOn = SavedSplit_;

    if (RestorePart_)
      _Multipart = false;
  }
  else if (_QuestionType == DIVISION)
  {
    int x = 0;
    int LastDiv_;
    int NumDiv_;
    int HasRem_ = _Random->GiveRandom(100);
    do _Dividend = _Random->GiveRandom(_HighValue);
    while (_Dividend == 0);
    bool Prime_ = IsPrime(_Dividend, LastDiv_);
    bool RestorePart_ = true;
    if (_Multipart)
      RestorePart_ = false;

    if (_Dividend > 1 && !Prime_)
    {
      do
      {
        ++x;
        do _Divisor = _Random->GiveRandom(_Dividend);
        while (_Divisor == 0);

        _Remainder = _Dividend % _Divisor;
        if (_Divisor == 1)
          break;
      }
      while (x < 500 && ((!RemainderAllowed(HasRem_) || _NoRemSet) && _Remainder > 0));

      if (x == 500)
      {
        _Divisor = LastDiv_;
        _Remainder = 0;
      }
    }
    else
    {
      if (_Dividend > 1)
      {
        NumDiv_ = _Random->GiveRandom(2);
        _Divisor = NumDiv_ ? _Dividend:1;
      }
      else
        _Divisor = 1;

      _Remainder = 0;
    }

    _Quotient = _Dividend / _Divisor;
    if (_SplitOn == 1)
    {
      if (_SplitOp == ADDITION)
        _SplitVal = _Random->GiveRandom(_Divisor);
      else if (_SplitOp == SUBTRACTION)
      {
        _SplitVal = _Random->GiveRandom(_HighValue - _Divisor);
        _SplitVal += _Divisor;
      }
    }
    else if (_SplitOn == 2)
    {
      _SplitOn = 0;
      SavedSplit_ = 2;
    }

    _FinalResult = _Quotient;
    _FinalResult2 = _Remainder;
    FillQuestion(_AppendSet);

    if (SavedSplit_)
      _SplitOn = SavedSplit_;

    if (RestorePart_)
      _Multipart = false;
  }
  else if (_QuestionType == ADDSUB)
  {
    int SavedType_ = _QuestionType;
    bool RestorePart_ = true;
    bool SavedMultiPart_ = _Multipart;
    _QuestionType = ADDITION;
    if (_Multipart)
      RestorePart_ = false;
    else
      SetMultipart(true);
    GiveQuestion();

    SetAppend(true);
    SetSubtrahend(_Sum);
    _QuestionType = SUBTRACTION;
    if (_Multipart)
      RestorePart_ = false;
    else
      SetMultipart(true);
    GiveQuestion();

    _QuestionType = SavedType_;
    _SubtrahendSet = false;
    _AppendSet = false;
    if (RestorePart_ || !SavedMultiPart_)
      _Multipart = false;
  }
  else if (_QuestionType == SUBADD)
  {
    int SavedType_ = _QuestionType;
    bool RestorePart_ = true;
    bool SavedMultiPart_ = _Multipart;
    _QuestionType = SUBTRACTION;
    if (_Multipart)
      RestorePart_ = false;
    else
      SetMultipart(true);
    GiveQuestion();

    SetAppend(true);
    SetAddend(_Difference);
    _QuestionType = ADDITION;
    if (_Multipart)
      RestorePart_ = false;
    else
      SetMultipart(true);
    GiveQuestion();

    _QuestionType = SavedType_;
    _AddendSet = false;
    _AppendSet = false;
    if (RestorePart_ || !SavedMultiPart_)
      _Multipart = false;
  }
  else if (_QuestionType == MULTADD)
  {
    int SavedType_ = _QuestionType;
    bool RestorePart_ = true;
    bool SavedMultiPart_ = _Multipart;
    _QuestionType = MULTIPLICATION;
    if (_Multipart)
      RestorePart_ = false;
    else
      SetMultipart(true);
    GiveQuestion();

    if (_SplitOn != 1)
    {
      SetAppend(true);
      SetAddend(_Product);
      _QuestionType = ADDITION;
      if (_Multipart)
        RestorePart_ = false;
      else
        SetMultipart(true);
      GiveQuestion();

      _AddendSet = false;
      _AppendSet = false;
    }

    _QuestionType = SavedType_;
    if (RestorePart_ || !SavedMultiPart_)
      _Multipart = false;
  }
  else if (_QuestionType == MULTSUB)
  {
    int SavedType_ = _QuestionType;
    bool RestorePart_ = true;
    bool SavedMultiPart_ = _Multipart;
    _QuestionType = MULTIPLICATION;
    if (_Multipart)
      RestorePart_ = false;
    else
      SetMultipart(true);
    GiveQuestion();

    if (_SplitOn != 1)
    {
      SetAppend(true);
      SetSubtrahend(_Product);
      _QuestionType = SUBTRACTION;
      if (_Multipart)
        RestorePart_ = false;
      else
        SetMultipart(true);
      GiveQuestion();

      _SubtrahendSet = false;
      _AppendSet = false;
    }

    _QuestionType = SavedType_;
    if (RestorePart_ || !SavedMultiPart_)
      _Multipart = false;
  }
  else if (_QuestionType == DIVADD)
  {
    SetNoRem(true);
    bool RestorePart_ = true;
    bool SavedMultiPart_ = _Multipart;
    int SavedType_ = _QuestionType;
    _QuestionType = DIVISION;
    if (_Multipart)
      RestorePart_ = false;
    else
      SetMultipart(true);
    GiveQuestion();

    if (_SplitOn != 1)
    {
      SetAppend(true);
      SetAddend(_Quotient);
      _QuestionType = ADDITION;
      if (_Multipart)
        RestorePart_ = false;
      else
        SetMultipart(true);
      GiveQuestion();

      _AddendSet = false;
      _AppendSet = false;
    }

    _QuestionType = SavedType_;
    SetNoRem(false);

    if (RestorePart_ || !SavedMultiPart_)
      _Multipart = false;
  }
  else if (_QuestionType == DIVSUB)
  {
    SetNoRem(true);
    bool RestorePart_ = true;
    bool SavedMultiPart_ = _Multipart;
    int SavedType_ = _QuestionType;
    _QuestionType = DIVISION;
    if (_Multipart)
      RestorePart_ = false;
    else
      SetMultipart(true);
    GiveQuestion();

    if (_SplitOn != 1)
    {
      SetAppend(true);
      SetSubtrahend(_Quotient);
      _QuestionType = SUBTRACTION;
      if (_Multipart)
        RestorePart_ = false;
      else
        SetMultipart(true);
      GiveQuestion();

      _SubtrahendSet = false;
      _AppendSet = false;
    }

    _QuestionType = SavedType_;
    SetNoRem(false);

    if (RestorePart_ || !SavedMultiPart_)
      _Multipart = false;
  }
  else if (_QuestionType == MULTADDSUB)
  {
    int SavedType_ = _QuestionType;
    _QuestionType = MULTADD;
    SetMultipart(true);
    GiveQuestion();

    SetAppend(true);
    SetSubtrahend(_Sum);
    _QuestionType = SUBTRACTION;
    SetMultipart(true);
    GiveQuestion();

    _QuestionType = SavedType_;
    _SubtrahendSet = false;
    _AppendSet = false;
    _Multipart = false;
  }
  else if (_QuestionType == DIVADDSUB)
  {
    SetNoRem(true);
    int SavedType_ = _QuestionType;
    _QuestionType = DIVADD;
    SetMultipart(true);
    GiveQuestion();

    SetAppend(true);
    SetSubtrahend(_Sum);
    _QuestionType = SUBTRACTION;
    SetMultipart(true);
    GiveQuestion();

    _QuestionType = SavedType_;
    _SubtrahendSet = false;
    SetNoRem(false);
    _AppendSet = false;
    _Multipart = false;
  }
  else if (_QuestionType == MULTSUBADD)
  {
    int SavedType_ = _QuestionType;
    _QuestionType = MULTSUB;
    SetMultipart(true);
    GiveQuestion();

    SetAppend(true);
    SetAddend(_Difference);
    _QuestionType = ADDITION;
    SetMultipart(true);
    GiveQuestion();

    _QuestionType = SavedType_;
    _AddendSet = false;
    _AppendSet = false;
    _Multipart = false;
  }
  else if (_QuestionType == DIVSUBADD)
  {
    SetNoRem(true);
    int SavedType_ = _QuestionType;
    _QuestionType = DIVSUB;
    SetMultipart(true);
    GiveQuestion();

    SetAppend(true);
    SetAddend(_Difference);
    _QuestionType = ADDITION;
    SetMultipart(true);
    GiveQuestion();

    _QuestionType = SavedType_;
    _AddendSet = false;
    SetNoRem(false);
    _AppendSet = false;
    _Multipart = false;
  }

  else if (_QuestionType == MULTADDpSUB)
  {
    int SavedType_ = _QuestionType;
    _QuestionType = MULTADD;
    _SplitOn = 1;
    _SplitOp = ADDITION;
    SetMultipart(true);
    GiveQuestion();

    _SplitOn = 0;
    SetAppend(true);
    SetSubtrahend(_Product);
    _QuestionType = SUBTRACTION;
    SetMultipart(true);
    GiveQuestion();

    _QuestionType = SavedType_;
    _SubtrahendSet = false;
    _AppendSet = false;
    _Multipart = false;
    _SplitOn = 0;
  }
  else if (_QuestionType == DIVADDpSUB)
  {
    SetNoRem(true);
    int SavedType_ = _QuestionType;
    _QuestionType = DIVADD;
    _SplitOn = 1;
    _SplitOp = ADDITION;
    SetMultipart(true);
    GiveQuestion();

    _SplitOn = 0;
    SetAppend(true);
    SetSubtrahend(_Quotient);
    _QuestionType = SUBTRACTION;
    SetMultipart(true);
    GiveQuestion();

    _QuestionType = SavedType_;
    _SubtrahendSet = false;
    SetNoRem(false);
    _AppendSet = false;
    _Multipart = false;
    _SplitOn = 0;
  }
  else if (_QuestionType == MULTSUBpADD)
  {
    int SavedType_ = _QuestionType;
    _QuestionType = MULTSUB;
    _SplitOn = 1;
    _SplitOp = SUBTRACTION;
    SetMultipart(true);
    GiveQuestion();

    _SplitOn = 0;
    SetAppend(true);
    SetAddend(_Product);
    _QuestionType = ADDITION;
    SetMultipart(true);
    GiveQuestion();

    _QuestionType = SavedType_;
    _AddendSet = false;
    _AppendSet = false;
    _Multipart = false;
    _SplitOn = 0;
  }
  else if (_QuestionType == DIVSUBpADD)
  {
    SetNoRem(true);
    int SavedType_ = _QuestionType;
    _QuestionType = DIVSUB;
    _SplitOn = 1;
    _SplitOp = SUBTRACTION;
    SetMultipart(true);
    GiveQuestion();

    _SplitOn = 0;
    SetAppend(true);
    SetAddend(_Quotient);
    _QuestionType = ADDITION;
    SetMultipart(true);
    GiveQuestion();

    _QuestionType = SavedType_;
    _AddendSet = false;
    SetNoRem(false);
    _AppendSet = false;
    _Multipart = false;
    _SplitOn = 0;
  }

  else if (_QuestionType == MULTADDSUBp)
  {
    int SavedType_ = _QuestionType;
    _QuestionType = MULTADD;
    _SplitOn = 2;
    _SplitOp = SUBTRACTION;
    SetMultipart(true);
    GiveQuestion();

    _QuestionType = SavedType_;
    _SubtrahendSet = false;
    _AppendSet = false;
    _Multipart = false;
    _SplitOn = 0;
  }
  else if (_QuestionType == DIVADDSUBp)
  {
    SetNoRem(true);
    int SavedType_ = _QuestionType;
    _QuestionType = DIVADD;
    _SplitOn = 2;
    _SplitOp = SUBTRACTION;
    SetMultipart(true);
    GiveQuestion();

    _QuestionType = SavedType_;
    _SubtrahendSet = false;
    SetNoRem(false);
    _AppendSet = false;
    _Multipart = false;
    _SplitOn = 0;
  }
  else if (_QuestionType == MULTSUBADDp)
  {
    int SavedType_ = _QuestionType;
    _QuestionType = MULTSUB;
    _SplitOn = 2;
    _SplitOp = ADDITION;
    SetMultipart(true);
    GiveQuestion();

    _QuestionType = SavedType_;
    _AddendSet = false;
    _AppendSet = false;
    _Multipart = false;
    _SplitOn = 0;
  }
  else if (_QuestionType == DIVSUBADDp)
  {
    SetNoRem(true);
    int SavedType_ = _QuestionType;
    _QuestionType = DIVSUB;
    _SplitOn = 2;
    _SplitOp = ADDITION;
    SetMultipart(true);
    GiveQuestion();

    _QuestionType = SavedType_;
    _AddendSet = false;
    SetNoRem(false);
    _AppendSet = false;
    _Multipart = false;
    _SplitOn = 0;
  }

  else if (_QuestionType == FRACTION_ADD)
  {
    int SavedType_ = _QuestionType;
    MakeFracHighValue();
    _QuestionType = ADDITION;
    SetMultipart(true);
    do GiveQuestion();
    while (_Addend == 0 || _Adder == 0);

    _Denom1 = _Addend;
    _Numer1 = _Random->GiveRandom(_Denom1);
    _Numer1++;

    _Denom2 = _Adder;
    _Numer2 = _Random->GiveRandom(_Denom2);
    _Numer2++;

    _ResNumer = (_Numer1 * _Denom2) + (_Numer2 * _Denom1);
    _ResDenom = _Denom1 * _Denom2;
    _SumFlt = double(_ResNumer) / _ResDenom;
    _Multipart = false;
    _QuestionType = SavedType_;
    Reduce();
    FillFracQuestion(false);
    RestoreHighValue();
    _Multipart = false;
  }
  else if (_QuestionType == FRACTION_SUB)
  {
    int SavedType_ = _QuestionType;
    MakeFracHighValue();
    _QuestionType = SUBTRACTION;
    SetMultipart(true);
    do GiveQuestion();
    while (_Subtrahend == 0 || _Subtractor == 0);

    _Denom1 = _Subtractor;
    _Numer1 = _Random->GiveRandom(_Denom1);
    _Numer1++;

    _Denom2 = _Subtrahend;
    _Numer2 = _Random->GiveRandom(_Numer1);
    _Numer2++;

    _ResNumer = (_Numer1 * _Denom2) - (_Numer2 * _Denom1);
    _ResDenom = _Denom1 * _Denom2;
    _DifferenceFlt = double(_ResNumer) / _ResDenom;
    _Multipart = false;
    _QuestionType = SavedType_;
    Reduce();
    FillFracQuestion(false);
    RestoreHighValue();
    _Multipart = false;
  }
  else if (_QuestionType == FRACTION_MULT)
  {
    int SavedType_ = _QuestionType;
    MakeFracHighValue();
    _QuestionType = ADDITION;
    SetMultipart(true);
    do GiveQuestion();
    while (_Addend == 0 || _Adder == 0);

    _Denom1 = _Addend;
    _Numer1 = _Random->GiveRandom(_Denom1);
    _Numer1++;

    _Denom2 = _Adder;
    _Numer2 = _Random->GiveRandom(_Denom2);
    _Numer2++;

    _ResNumer = _Numer1 * _Numer2;
    _ResDenom = _Denom1 * _Denom2;
    _ProductFlt = double(_ResNumer) / _ResDenom;
    _Multipart = false;
    _QuestionType = SavedType_;
    Reduce();
    FillFracQuestion(false);
    RestoreHighValue();
    _Multipart = false;
  }
  else if (_QuestionType == FRACTION_DIV)
  {
    int SavedType_ = _QuestionType;
    MakeFracHighValue();
    _QuestionType = ADDITION;
    SetMultipart(true);
    do GiveQuestion();
    while (_Addend == 0 || _Adder == 0);

    _Denom1 = _Addend;
    _Numer1 = _Random->GiveRandom(_Denom1);
    _Numer1++;

    _Numer2 = _Adder;
    _Denom2 = _Random->GiveRandom(_Numer2);
    _Denom2++;

    _ResNumer = _Numer1 * _Denom2;
    _ResDenom = _Denom1 * _Numer2;
    _QuotientFlt = double(_ResNumer) / _ResDenom;
    _Multipart = false;
    _QuestionType = SavedType_;
    Reduce();
    FillFracQuestion(false);
    RestoreHighValue();
    _Multipart = false;
  }

  if (!_Multipart)
  {
    if (_QuestionType == FRACTION_ADD ||
        _QuestionType == FRACTION_SUB ||
        _QuestionType == FRACTION_MULT ||
        _QuestionType == FRACTION_DIV)
      FillFracAnswer();
    else
      FillAnswer();

    _Index++;
  }

  if (IsMixed_)
    _QuestionType = IsMixed_;
  else if (!_Multipart && _QuestionType != MIXED)
    _GivenQuestionType = _QuestionType;

  return _QuestionStr;
}

/******************************************************************************/
void ExamGenerator::MakeFracHighValue()
{
  int Limit_ = _DigitLimit / 2;
  if (!Limit_)
    Limit_ = 1;

  _SavedHighValue = _HighValue;
  _HighValue = int(::ceil(::pow(10L, Limit_)));
}

/******************************************************************************/
void ExamGenerator::RestoreHighValue()
{
  _HighValue = _SavedHighValue;
}

/******************************************************************************/
int ExamGenerator::QuestionNumber() const
{
  return _Index;
}

/******************************************************************************/
bool ExamGenerator::IsFraction() const
{
  return (_QuestionType == FRACTION_ADD ||
          _QuestionType == FRACTION_SUB ||
          _QuestionType == FRACTION_MULT ||
          _QuestionType == FRACTION_DIV);
}

/******************************************************************************/
bool ExamGenerator::AllQuestionsDone() const
{
  return (_Index >= _Max);
}

/******************************************************************************/
int ExamGenerator::FinalAnswer() const
{
  return _FinalResult;
}

/******************************************************************************/
int ExamGenerator::FinalNumer() const
{
  return _ResNumer;
}

/******************************************************************************/
int ExamGenerator::FinalDenom() const
{
  return _ResDenom;
}

/******************************************************************************/
int ExamGenerator::Remainder() const
{
  return _FinalResult2;
}

/******************************************************************************/
const char* ExamGenerator::FinalFraction() const
{
  if (_QuestionType == FRACTION_ADD ||
      _QuestionType == FRACTION_SUB ||
      _QuestionType == FRACTION_MULT ||
      _QuestionType == FRACTION_DIV)
    return _AnswerStr;

  return NULL;
}

/******************************************************************************/
const char* ExamGenerator::QuestionType() const
{
  return
  (
     (_GivenQuestionType == ADDITION)       ? "Addition":
     (_GivenQuestionType == SUBTRACTION)    ? "Subtraction":
     (_GivenQuestionType == MULTIPLICATION) ? "Multiplication":
     (_GivenQuestionType == DIVISION)       ? "Division":
     (_GivenQuestionType == ADDSUB)         ? "Addition, Subtraction":
     (_GivenQuestionType == SUBADD)         ? "Subtraction, Addition":
     (_GivenQuestionType == MULTADD)        ? "Multiplication, Addition":
     (_GivenQuestionType == MULTSUB)        ? "Multiplication, Subtraction":
     (_GivenQuestionType == DIVADD)         ? "Division, Addition":
     (_GivenQuestionType == DIVSUB)         ? "Division, Subtraction":
     (_GivenQuestionType == MULTADDSUB)     ? "Multiplication, Addition, Subtraction":
     (_GivenQuestionType == DIVADDSUB)      ? "Division, Addition, Subtraction":
     (_GivenQuestionType == MULTSUBADD)     ? "Multiplication, Subtraction, Addition":
     (_GivenQuestionType == DIVSUBADD)      ? "Division, Subtraction, Addition":

     (_GivenQuestionType == MULTADDpSUB)    ? "Multiplication, (Addition), Subtraction":
     (_GivenQuestionType == DIVADDpSUB)     ? "Division, (Addition), Subtraction":
     (_GivenQuestionType == MULTSUBpADD)    ? "Multiplication, (Subtraction), Addition":
     (_GivenQuestionType == DIVSUBpADD)     ? "Division, (Subtraction), Addition":

     (_GivenQuestionType == MULTADDSUBp)    ? "Multiplication, Addition, (Subtraction)":
     (_GivenQuestionType == DIVADDSUBp)     ? "Division, Addition, (Subtraction)":
     (_GivenQuestionType == MULTSUBADDp)    ? "Multiplication, Subtraction, (Addition)":
     (_GivenQuestionType == DIVSUBADDp)     ? "Division, Subtraction, (Addition)":

     (_GivenQuestionType == FRACTION_ADD)   ? "Fraction Addition":
     (_GivenQuestionType == FRACTION_SUB)   ? "Fraction Subtraction":
     (_GivenQuestionType == FRACTION_MULT)  ? "Fraction Multiplication":
     (_GivenQuestionType == FRACTION_DIV)   ? "Fraction Division":"None"
  );
}

/******************************************************************************/
const char* ExamGenerator::FinalNumber() const
{
  if (_QuestionType != FRACTION_ADD &&
      _QuestionType != FRACTION_SUB &&
      _QuestionType != FRACTION_MULT &&
      _QuestionType != FRACTION_DIV)
    return _AnswerStr;

  return NULL;
}

/******************************************************************************/
#if EXAMGENERATOR_DEBUG
int main(int argc, char* argv[])
{
  ExamGenerator* Gen = new ExamGenerator();
  Gen->GenerateProblems(50, ExamGenerator::MIXED, 4);
  const char* Question_;
  Gen->SetDivRemainderPercentage(50);

  while (!Gen->AllQuestionsDone())
  {
    Question_ = Gen->GiveQuestion();

    cout <<endl;
    cout <<Gen->QuestionType() <<endl;
    cout <<"Q" <<Gen->QuestionNumber() <<": " <<Question_ <<endl;
    cout <<"A" <<Gen->QuestionNumber() <<": ";

    if (Gen->IsFraction())
      cout <<Gen->FinalFraction() <<endl;
    else
      cout <<Gen->FinalNumber() <<endl;
  }

  delete Gen;
  return 1;
}
#endif
/******************************************************************************/
