#include <fstream>
#include <iostream>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <ctype.h>

#include "randomizer.h"

#define EXAMGENERATOR_DEBUG        0
#define EXAMGENERATOR_GUI_DEBUG    0

#define USES_STD                   0

#if USES_STD
  #define OSTREAM std::ostream
  #define ISTREAM std::istream
#else
  #define OSTREAM ostream
  #define ISTREAM istream
#endif

using namespace std;

/******************************************************************************/
class ExamGenerator
{
  public:
    enum
    {
      ADDITION       = 0,
      SUBTRACTION    = 1,
      MULTIPLICATION = 2,
      DIVISION       = 3,

      ADDSUB         = 4,
      SUBADD         = 5,
      MULTADD        = 6,
      MULTSUB        = 7,
      DIVADD         = 8,
      DIVSUB         = 9,
      MULTADDSUB     = 10,
      DIVADDSUB      = 11,
      MULTSUBADD     = 12,
      DIVSUBADD      = 13,

      MULTADDpSUB    = 14,
      DIVADDpSUB     = 15,
      MULTSUBpADD    = 16,
      DIVSUBpADD     = 17,

      MULTADDSUBp    = 18,
      DIVADDSUBp     = 19,
      MULTSUBADDp    = 20,
      DIVSUBADDp     = 21,

      FRACTION_ADD   = 22,
      FRACTION_SUB   = 23,
      FRACTION_MULT  = 24,
      FRACTION_DIV   = 25,

      MIXED          = 26
    };

  private:
    int _Max;
    int _Index;
    int _DigitLimit;
    int _HighValue;
    int _SavedHighValue;
    unsigned _Seed;

    Randomizer<long double>* _Random;
    int _QuestionType;
    int _GivenQuestionType;
    char* _QuestionStr;
    int _FinalResult;
    int _FinalResult2;
    char* _AnswerStr;

    int _Numer1;
    int _Numer2;
    int _Denom1;
    int _Denom2;
    int _ResNumer;
    int _ResDenom;

    int _Dividend;
    int _Divisor;
    int _Quotient;
    int _Remainder;
    int _RemainderPercent;
    double _QuotientFlt;

    int _Addend;
    int _Adder;
    int _Sum;
    double _SumFlt;

    int _Subtrahend;
    int _Subtractor;
    int _Difference;
    double _DifferenceFlt;

    int _Multiplicand;
    int _Multiplier;
    int _Product;
    double _ProductFlt;

    bool _AddendSet;
    bool _SubtrahendSet;
    bool _NoRemSet;
    bool _AppendSet;
    bool _Multipart;
    int _SplitOn;
    int _SplitOp;
    int _SplitVal;

    void SetAddend(int Add_);
    void SetSubtrahend(int Sub_);
    void SetNoRem(bool NoRem_)
        { _NoRemSet = NoRem_; }
    void SetAppend(bool Append_)
        { _AppendSet = Append_; }
    void SetMultipart(bool Multi_)
        { _Multipart = Multi_; }

    bool RemainderAllowed(int Val_) const;
    bool IsPrime(int Num_, int& Div_) const;
    void FillQuestion(bool Append_);
    void FillFracQuestion(bool Append_);
    void MakeFracHighValue();
    void RestoreHighValue();

    void FillAnswer();
    void FillFracAnswer();
    void Reduce();
    void Clear();

  public:
    ExamGenerator();
    ExamGenerator(int Max_);
    ~ExamGenerator();

    void SetDivRemainderPercentage(int Val_);
    void GenerateProblems(int Max_, int Type_, int DigitLimit_);
    const char* GiveQuestion();
    bool AllQuestionsDone() const;
    bool IsFraction() const;

    int QuestionNumber() const;
    int FinalAnswer() const;
    int FinalNumer() const;
    int FinalDenom() const;
    int Remainder() const;

    const char* QuestionType() const;
    const char* FinalFraction() const;
    const char* FinalNumber() const;
};

