// File: common.cpp
// Purpose: commonly used functions of the project
//
#include "common.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
CommonFncs::CommonFncs()
{
  _WithMsg = _WithLog = false;
  _LineCnt = NULL;
  _MaxLineCnt = DEFAULT_MAX_LINE_CNT;
  _Set = NULL;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
CommonFncs::~CommonFncs()
{
  if (!_Set && _LineCnt)
    delete _LineCnt;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CommonFncs::GetLineCount() const
{
  if (_LineCnt)
    return (*_LineCnt);

  return 0;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CommonFncs::GetMaxLineCount() const
{
  return _MaxLineCnt;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CommonFncs::SetTraceWithMsg(bool Flag_)
{
  _WithMsg = Flag_;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CommonFncs::SetTraceWithLog(bool Flag_, int MaxLineCnt_, int* LineCnt_)
{
  _WithLog = Flag_;
  
  if (MaxLineCnt_)
    _MaxLineCnt = MaxLineCnt_;
  else
    _MaxLineCnt = DEFAULT_MAX_LINE_CNT;

  if (!_Set && _LineCnt)
    delete _LineCnt;
  
  if (LineCnt_)
    _Set = _LineCnt = LineCnt_;
  else
  {
    _LineCnt = new int;
    _Set = NULL;
    *_LineCnt = 0;
  }
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CommonFncs::StartTrace(QFile& DbgFile_, const QString& Caption_, const QString& FncName_, const QString& Args_)
{
  QString Text_("Starting: ");
  Text_ += FncName_;
  Text_ += Args_;

  if (_WithMsg)
    QMessageBox::warning(0, Caption_, Text_,
                         QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);

  if (_WithLog)
  {
    QTextStream dbgout(&DbgFile_);
    dbgout.setEncoding(QTextStream::UnicodeUTF8);
    dbgout <<Text_ <<endl;
    if (_LineCnt)
      ++(*_LineCnt);

    if (_LineCnt && (*_LineCnt >= _MaxLineCnt))
    {
      dbgout.device()->reset();
      *_LineCnt = 0;
    }
  }
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CommonFncs::EndTrace(QFile& DbgFile_, const QString & Caption_, const QString & FncName_, const QString & Args_ )
{
  QString Text_("Ending: ");
  Text_ += FncName_;
  Text_ += Args_;

  if (_WithMsg)
    QMessageBox::warning(0, Caption_, Text_,
                         QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);

  if (_WithLog)
  {
    QTextStream dbgout(&DbgFile_);
    dbgout.setEncoding(QTextStream::UnicodeUTF8);
    dbgout <<Text_ <<endl;
    if (_LineCnt)
      ++(*_LineCnt);

    if (_LineCnt && (*_LineCnt >= _MaxLineCnt))
    {
      dbgout.device()->reset();
      *_LineCnt = 0;
    }    
  }
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CommonFncs::ArgTrace(QFile& DbgFile_, const QString & Caption_, const QString & ArgName_, const QString & Args_ )
{
  QString Text_(ArgName_);
  Text_ += " = ";
  Text_ += Args_;

  if (_WithMsg)
    QMessageBox::warning(0, Caption_, Text_,
                         QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);

  if (_WithLog)
  {
    QTextStream dbgout(&DbgFile_);
    dbgout.setEncoding(QTextStream::UnicodeUTF8);
    dbgout <<Text_ <<endl;
    if (_LineCnt)
      ++(*_LineCnt);

    if (_LineCnt && (*_LineCnt >= _MaxLineCnt))
    {
      dbgout.device()->reset();
      *_LineCnt = 0;
    }    
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CommonFncs::OpenTextFile(const QString& Name_, QFlags<QIODevice::OpenModeFlag> Mode_, QFile& File_, const QString& AppName_)
{
  File_.resetStatus();
  File_.setName(Name_);
  
  if (!File_.open(Mode_))
  {
    ioError(File_, QObject::tr("Cannot open file %1 for writing"), AppName_);
    return false;
  }

  return true;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CommonFncs::OpenBinFile(const QString& Name_, QFlags<QIODevice::OpenModeFlag> Mode_, QFile& File_, const QString& AppName_)
{
  File_.resetStatus();
  File_.setName(Name_);
  
  if (!File_.open(Mode_))
  {
    ioError(File_, QObject::tr("Cannot open file %1 for writing"), AppName_);
    return false;
  }

  return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString CommonFncs::RootPath()
{
    QString Path_ = QDir::rootDirPath();
    
    #if defined(Q_OS_WIN32) & !defined(__linux__) & !defined(__unix__)      
      Path_ = Path_.left(2);
      Path_ += "\\";
    #endif

    return Path_;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CommonFncs::Error(const QFile& File_, const QString& Msg_, const QString& AppName_)
{
    QMessageBox::warning(0, QObject::tr("%1").arg(AppName_), Msg_.arg(File_.name()));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CommonFncs::ioError(const QFile& File_, const QString& Msg_, const QString& AppName_)
{
    Error(File_, Msg_ + ": " + File_.errorString(), AppName_);
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CommonFncs::CloseBlock(QByteArray& Block_, QDataStream& out, Q3Socket& Socket_)
{
    out.device()->at(0);
    out <<(Q_UINT16)(Block_.size() - sizeof(Q_UINT16));
    Socket_.writeBlock(Block_.data(), Block_.size());
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
Q_UINT8 CommonFncs::SendSignal(Q_UINT8 Signal_, QDataStream& out)
{
    out.setVersion(5);
    out <<(Q_UINT16)0 <<Signal_;
    return Signal_;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CommonFncs::CopyTxtFileToFile(const QString& SrceDir_, const QString& DestDir_,
                                   const QString& SrceName_, const QString& DestName_)
{
   QFile InFile_;
   QFile OutFile_;
   
   QString SrceFileName_;
   QString DestFileName_;

   SrceFileName_ = AppendPathToName(SrceFileName_, SrceDir_, SrceName_);
   DestFileName_ = AppendPathToName(DestFileName_, DestDir_, DestName_);
   
   OpenTextFile(SrceFileName_, IO_ReadOnly | IO_Translate, InFile_, "CommonFncs");
   OpenTextFile(DestFileName_, IO_WriteOnly | IO_Translate, OutFile_, "CommonFncs");
   
   QTextStream in(&InFile_);
   QTextStream out(&OutFile_);

   QString Buffer_;
   Buffer_ = in.read();
   out <<Buffer_;
   
   InFile_.close();
   OutFile_.close();
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CommonFncs::CopyTxtFile(const QString& SrceDir_, const QString& DestDir_, const QString& Name_)
{
   QFile InFile_;
   QFile OutFile_;
   
   QString SrceFileName_;
   QString DestFileName_;

   SrceFileName_ = AppendPathToName(SrceFileName_, SrceDir_, Name_);
   DestFileName_ = AppendPathToName(DestFileName_, DestDir_, Name_);
   
   OpenTextFile(SrceFileName_, IO_ReadOnly | IO_Translate, InFile_, "CommonFncs");
   OpenTextFile(DestFileName_, IO_WriteOnly | IO_Translate, OutFile_, "CommonFncs");
   
   QTextStream in(&InFile_);
   QTextStream out(&OutFile_);

   QString Buffer_;
   Buffer_ = in.read();
   out <<Buffer_;
   
   InFile_.close();
   OutFile_.close();
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CommonFncs::CopyBinFileToFile(const QString& SrceDir_, const QString& DestDir_,
                                   const QString& SrceName_, const QString& DestName_)
{
   QFile InFile_;
   QFile OutFile_;
   
   QString SrceFileName_;
   QString DestFileName_;

   SrceFileName_ = AppendPathToName(SrceFileName_, SrceDir_, SrceName_);
   DestFileName_ = AppendPathToName(DestFileName_, DestDir_, DestName_);
   
   OpenBinFile(SrceFileName_, IO_ReadOnly | IO_Translate, InFile_, "CommonFncs");
   OpenBinFile(DestFileName_, IO_WriteOnly | IO_Translate, OutFile_, "CommonFncs");

   QByteArray Buffer_ = InFile_.readAll();
   OutFile_.writeBlock(Buffer_.data(), Buffer_.size());
   
   InFile_.close();
   OutFile_.close();
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CommonFncs::CopyBinFile(const QString& SrceDir_, const QString& DestDir_, const QString& Name_)
{
   QFile InFile_;
   QFile OutFile_;
   
   QString SrceFileName_;
   QString DestFileName_;

   SrceFileName_ = AppendPathToName(SrceFileName_, SrceDir_, Name_);
   DestFileName_ = AppendPathToName(DestFileName_, DestDir_, Name_);
   
   OpenBinFile(SrceFileName_, IO_ReadOnly | IO_Translate, InFile_, "CommonFncs");
   OpenBinFile(DestFileName_, IO_WriteOnly | IO_Translate, OutFile_, "CommonFncs");

   QByteArray Buffer_ = InFile_.readAll();
   OutFile_.writeBlock(Buffer_.data(), Buffer_.size());
   
   InFile_.close();
   OutFile_.close();
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString& CommonFncs::AppendPathToName(QString& Result_, const QString & Path_, const QString & Filename_)
{
  Result_ = Path_;
  Result_ = Result_.stripWhiteSpace();

  if (Path_.length())
  {
    #if defined(Q_OS_WIN32) & !defined(__linux__) & !defined(__unix__)
      if (Result_.right(1) == "\\")
        Result_ += Filename_;
      else
      {
        Result_ += "\\";
        Result_ += Filename_;
      }
    #else
      if (Result_.right(1) == "/")
        Result_ += Filename_;
      else
      {
        Result_ += "/";
        Result_ += Filename_;
      }
    #endif
  }
  else
    Result_ = Filename_;

  return Result_;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString& CommonFncs::SecsToHms(QString& Str_, Q_ULONG Secs_)
{
  QString hh;
  QString mm;
  QString ss;

  hh = QString::number(Secs_ / 3600);
  mm = QString::number((Secs_ % 3600) / 60);
  ss = QString::number((Secs_ % 3600) % 60);

  if (hh.length() == 1)
    hh = QString("0") + hh;

  if (mm.length() == 1)
    mm = QString("0") + mm;

  if (ss.length() == 1)
    ss = QString("0") + ss;  

  Str_ = hh;
  Str_ += ":";
  Str_ += mm;
  Str_ += ":";
  Str_ += ss;

  return Str_;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

