#include "viewhelp.h"

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
HelpViewer::HelpViewer()
{
  _WebBrowserPath = "";
  ShowHelp();
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool HelpViewer::SelectPathToWebBrowser()
{
    QString CurPath_ = _WebBrowserPath;
    QString Start_;

    if (CurPath_.isEmpty() || CurPath_ == QString("."))
    {
      if (QFile::exists(DEFAULT_WEB_BROWSER))
        Start_ = DEFAULT_WEB_BROWSER;
      else
        Start_ = DEFAULT_APPS_DIR;
    }
    else
      Start_ = CurPath_;

    QString FilePath_ =
    Q3FileDialog::getOpenFileName
    (
      Start_,
      QString(APPS_FILE_FILTER),
      this,
      "GetWebPath",
      QString("Select File Path to Web Browser Program")
    );

    if (!FilePath_.isEmpty() && QFile::exists(FilePath_))
      _WebBrowserPath = FilePath_;
    else if (QFile::exists(DEFAULT_WEB_BROWSER))
      _WebBrowserPath = DEFAULT_WEB_BROWSER;

    return (!_WebBrowserPath.isEmpty());
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void HelpViewer::ShowHelp()
{
    QDir Dir_(QDir::currentDirPath());

    if (_WebBrowserPath.isEmpty())
    {
      QString FilePath_ = Dir_.absPath();
      FilePath_ += QString(QChar(QDir::separator()));
      FilePath_ += QString(WEB_BROWSER_PATH_FILE);

      QFile File_;

      if (QFile::exists(FilePath_))
      {
        File_.setName(FilePath_);

        if (!File_.open(IO_ReadOnly | IO_Translate))
        {
          if (SelectPathToWebBrowser())
          {
            ShowHelp();
            return;
          }
          else
          {
            QMessageBox::critical(0,
              QString(tr("Fatal Error")),
              QString(tr("Fatal Error: Could not start the web browser program.\n")),
              QString(tr("Quit")));
            exit( -1 );
          }
        }
        else
        {
          QTextStream fin(&File_);
          fin.setEncoding(QTextStream::UnicodeUTF8);

          QString Str_ = fin.readLine();
          Str_ = Str_.stripWhiteSpace();
          _WebBrowserPath = Str_;
          File_.close();

          if (!QFile::exists(_WebBrowserPath))
          {
            File_.setName(FilePath_);
            File_.open(IO_WriteOnly | IO_Translate);

            QTextStream fout(&File_);
            fout.setEncoding(QTextStream::UnicodeUTF8);
            fout.setDevice(&File_);

            Str_ = WEB_BROWSER_2ND_CHOICE;
            Str_ = Str_.stripWhiteSpace();

            if (QFile::exists(Str_))
            {
              _WebBrowserPath = Str_;
              fout <<_WebBrowserPath <<"\n";
              File_.close();
            }
            else
            {
              Str_ = WEB_BROWSER_3RD_CHOICE;
              Str_ = Str_.stripWhiteSpace();

              if (QFile::exists(Str_))
              {
                _WebBrowserPath = Str_;
                fout <<_WebBrowserPath <<"\n";
                File_.close();
              }
              else
              {
                Str_ = WEB_BROWSER_4TH_CHOICE;
                Str_ = Str_.stripWhiteSpace();

                if (QFile::exists(Str_))
                {
                  _WebBrowserPath = Str_;
                  fout <<_WebBrowserPath <<"\n";
                  File_.close();
                }
                else
                {
                  Str_ = WEB_BROWSER_5TH_CHOICE;
                  Str_ = Str_.stripWhiteSpace();

                  if (QFile::exists(Str_))
                  {
                    _WebBrowserPath = Str_;
                    fout <<_WebBrowserPath <<"\n";
                    File_.close();
                  }
                }
              }
            }
          }
        }
      }
      else
      {
        if (SelectPathToWebBrowser())
        {
          ShowHelp();
          return;
        }
        else
        {
          QMessageBox::critical(0,
            QString(tr("Fatal Error")),
            QString(tr("Fatal Error: Could not start the web browser program.\n")),
            QString(tr("Quit")));
          exit( -1 );
        }
      }
    }

    QString DocPath_ = Dir_.absPath();
    DocPath_.replace(QChar('/'), "\\");
    // DocPath_ += QString(QChar(QDir::separator()));
    DocPath_ += QString(DIRSEP);
    DocPath_ += QString(DOCUMENT_DIR);
    // DocPath_ += QString(QChar(QDir::separator()));
    DocPath_ += QString(DIRSEP);

    if (strncmp(QTextCodec::locale(), "en", 2) == 0)
      DocPath_ += QString(HELP_START_FILE);
    else
      DocPath_ += QString(HELP_START_FILE);

    // QProcess related code
    proc = new Q3Process( this );

    // Set up the command and arguments.
    // On the command line you would do:
    //   uic -tr i18n "small_dialog.ui"
    proc->addArgument(_WebBrowserPath);
    proc->addArgument(DocPath_);

    connect(proc, SIGNAL(processExited()),
            qApp, SLOT(quit()));

    if (!proc->start())
    {
      // error handling
      QMessageBox::critical(0,
        QString(tr("Fatal Error")),
        QString(tr("Fatal Error: Could not start the web browser program.\n")),
        QString(tr("Quit")));
      exit( -1 );
    }
}
